﻿using FMK.EFH.Contracts;
//using FMK.EFH.Contracts.PowerOfAttorneys.HandleCustomer;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Net.Http;
using System.Net.Http.Formatting;
using System.Threading.Tasks;

namespace FMK.EFH.API.Sample
{
    public class Program
    {
        private static void Main(string[] args)
        {
            Console.WriteLine("Fullmaktskollen - Kodexempel för integration med fullmaktskollens API");

            using (var client = ClientFactory.CreateForAgent())
            {
                #region GetSystemIdFromAPi

                Console.WriteLine("\nExempel för att hämta ropande systemid:");
                try
                {
                    Console.WriteLine("Hämtar systemid...");
                    var identity = ApiRequest.Process<ApiIdentity>(() => client.GetAsync(new Uri("api/v1/Authorization/Identity", UriKind.Relative)));
                    Console.WriteLine("Anropar API med systemid: " + identity.SystemId + ", " + identity.Type);
                }

                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }
                #endregion

                #region CreatePoa
                var poa = new POACreateRequest
                {
                    Agent = new POAAgent
                    {
                        Name = "Agent Name",
                        //Notera att detta organisationsnummer måste vara erat eget (hämtas från app.config)
                        OrganizationNumber = ConfigurationManager.AppSettings["FMK.EFH.API.AgentOrganizationNumber"]
                    },
                    Type = POAType.InformationPrivate,
                    ValidTo = DateTime.Now.AddDays(90),
                    Validity = POAValidityType.Timespan,
                    SignatureType = POASignatureType.Digital,
                    Signees = new[]
                    {
                        new POASignee
                        {
                            FirstName = "Bertil Gösta",
                            LastName = "Svéden",
                            CivicRegistrationNumber = "195006272354",
                            Email = "bertil.gosta-test@fullmaktskollen.se",
                            PhoneNumber = "+46731909000"
                        }
                    }
                };

                var signingCertificate = ClientFactory.GetClientCertificate();

                Console.WriteLine("\nExempel på skapande av fullmakt:");

                try
                {
                    Console.WriteLine("Skapar fullmakt...");
                    var poaIdentity = ApiRequest.Process<POAIdentity>(() => SendAsync(client, "api/v1/POA/Create", poa));
                    Console.WriteLine("Skapat fullmakt med ID: " + poaIdentity.POAId);
                }
                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }
                #endregion

                #region CreatePoaWithValidationError
                poa.Signees = new[]
                {
                    new POASignee
                    {
                        FirstName = "Bertil Gösta",
                        LastName = "Svéden",
                        CivicRegistrationNumber = "195006272354",
                        Email = "bertil.gosta-test@fullmaktskollen.se",
                        PhoneNumber = "08531909000" //Felaktigt telefonnummer enligt validering (medvetet).
                    }
                };

                Console.WriteLine("\nExempel på 400 bad request:");

                try
                {
                    Console.WriteLine("Skapar fullmakt...");
                    var poaIdentity2 = ApiRequest.Process<POAIdentity>(() => SendAsync(client, "api/v1/POA/Create", poa));
                    Console.WriteLine("Skapat fullmakt med ID: " + poaIdentity2.POAId); // Skall aldrig komma hit pga BadRequest 400 (felaktigt telefonnummer)
                }

                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }

                catch (Exception se)
                {
                    if (se.InnerException != null)
                        Console.WriteLine(se.InnerException.Message);
                    else
                        Console.WriteLine(se.Message);
                }
                #endregion

                #region SearchPoasForAgent
                var request = new POASearchRequest
                {
                    Filter = "195006272354",
                    Scope = POASearchFilterScope.RegistrationNumber,
                    Skip = 0,
                    Count = 10,
                    OrderBy = POASearchOrderBy.Name,
                    SortDirection = POASearchDirection.Ascending
                };

                Console.WriteLine("\nExempel på fullmaktssökning:");
                try
                {
                    Console.WriteLine("Söker fullmakt/fullmakter...");
                    var searchResponse = ApiRequest.Process<POASearchResponse>(() => SendAsync(client, "api/v1/POA/Search", request));
                    Console.WriteLine("Hittade: " + searchResponse.TotalCount + " fullmakt(er).");
                }
                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }
                #endregion
            }

            using (var client = ClientFactory.CreateForPresenteeParty())
            {
                #region UnsubscribeCustomer

                Console.WriteLine("\nExempel på terminering av prenumeration:");

                try
                {
                    var request = new GetPOAClientRequest
                    {
                        RegistrationNo = "198007062311"
                    };

                    Console.WriteLine("Laddar fullmakt/fullmakter...");
                    var getResponse = ApiRequest.Process<List<GetPOAResponse>>(() => SendAsync(client, "api/v1/POA/GetPOAsForCustomer", request));
                    Console.WriteLine("Hittade: " + getResponse.Count + " fullmakt(er).");
                }
                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }

                try
                {
                    var request = new UnsubscribeCustomerRequest
                    {
                        RegistrationNo = "198007062311"
                    };

                    Console.WriteLine("Avslutar prenumeration på fullmakter för " + request.RegistrationNo);
                    ApiRequest.Process(() => SendAsync(client, "api/v1/PresenteeParty/UnsubscribeCustomer", request));
                    Console.WriteLine("Prenumeration avslutad.");
                }
                catch (ApiException e)
                {
                    Console.WriteLine("HTTP status code: " + e.Model.StatusCode);
                    foreach (var error in e.Model.Details)
                    {
                        Console.WriteLine(error.Path + ", " + error.ErrorId + ": " + error.Error);
                    }
                }

                #endregion

                Console.WriteLine("\n\nTryck på valfri knapp för att avsluta.");
                Console.ReadLine();
            }
        }

        public static Task<HttpResponseMessage> SendAsync<T>(HttpClient client, string uri, T obj)
        {
            var msg = new HttpRequestMessage(HttpMethod.Post, uri)
            {
                Method = HttpMethod.Post,
                Content = new ObjectContent<T>(obj, GetResponseFormatter())
            };
            return client.SendAsync(msg);
        }

        private static MediaTypeFormatter GetResponseFormatter()
        {
            var xmlFormatter = new XmlMediaTypeFormatter();
            return xmlFormatter;
        }

    }
}